
/*********************************************************************************                                                                                   *
FILE NAME      SPI_26A.c
DESCRIPTION:  Main routine for SPI sample code running on M16C/26A MCU, UART0, Mode3.
This program uses UART0.  The Chip Select for the flash device is set to P6_0.  The

The UART configuration has
the baud rate generator set for divide by 2 mode.  The MCU clock is set for 20 MHz (the
maximum for the M16C Tiny family) which gives a max 10 MHz SPI clock rate.  The clock
speed was verified using a scope

This program sets up UART0 in SPI master mode.  The configuration allows setting
the peripheral up for any of the 2 SPI modes (1 or 3) and allows using 3 wire (single slave,
no slave select needed) or 4 wire (port P6_0 and P10_4 are used for slave selects
to be consistent with the hardware manual example.  However, any available port pins
can be used for slave select outputs).  Write Protect pin is controlled by port P7_5,
and the code unprotects during command sequences.

The Transfer Clock (baud rate) was limited to 769230 (u0brg=0x0c), only because in this design
(not the RSK or SKP) the MCU ran at 5V, the M25P40 ran at 3V, and there were two transistors
used to raise the serial output (Q) up to 5V so the M16C RxD0 pin would recognize a high level.
The impedance/capacitance in the circuit was high enough to limit the transmission speed below the M16C max
possible - 10Mbps.

Copyright   : 2005 Renesas Technology Corporation.
All Rights Reserved
**********************************************************************************/

/**********************************************************************************
Revision History
DD.MM.YYYY OSO-UID Description
11.08.2005 RTA-MGF First Release
05.10.2007 RTA-JMB Revised to support HTSdemo project

**********************************************************************************/

/**********************************************************************************
System Includes
***********************************************************************************/


/**********************************************************************************
User Includes
***********************************************************************************/
#include "spi.h"
/*lint -save -e* */
#include "sfr26a.h"
/*lint -restore */
#include "HTSdemodef.h"

/* ************************* Global variables ********************************/
static unsigned char dummy;

/*****************************************************************************
Name:    SPI_open
Parameters:  None
Returns:  None
Description: Configure uart0 for SPI, Mode 3, assert slave select
*****************************************************************************/
void SPI_open(void)
{
  /* disable slave select */
  FLASH_SCS = 1;

  /* set master clock bit rate */
  /* Transfer clock = f1SIO/(2(n+1)), n=u0brg */
  u0brg = 1;

  /* UART0 transmit/receive control register 0 */
  u0c0 = 0x90;   /*  10010000  f1 clock source,MSB first, CTS/RTS disabled, CMOS output */

  /* UART0 transmit/receive control register 1 */
  u0c1 = 0x00;   /*    00000000 disable transmit and receive   */

  /* UART0 transmit/receive mode register*/
  u0mr = 0x01;  /* 00000001   synch serial,not reversed    */


  /* UART0 transmit/receive control register 1 */
  u0c1 = 0;     // UART0 transmit/receive control register 1
    /* 00000000  disable transmit and receive
    b0  TE      Transmit enable bit
    b1  TI      Transmit buffer empty flag
    b2  RE      Receive enable bit
    b3  RI      Receive complete flag
    b5:b7      Reserved, set to 0       */


  /* using UART0/1, the two different modes of SPI are achieved by changing the clock
  polarity, while clock phase is not changable.  The default mode set below is Mode 3,
  to use mode 1 comment out the mode 3 setup and uncomment mode 1*/

  /* Mode 1 - data shifted on rising edge, latched on falling edge.  Clock will rest low */
  //   ckpol_u0c0 = 1;

  /* Mode 3 - data shifted on falling edge, latched on rising edge.  Clock will rest high */
  ckpol_u0c0 = 0 ;

  /* perform dummy read to empty receive buffer */
  dummy = u0rbl;
  /* enable SPI */
  re_u0c1 = 1;
  te_u0c1 = 1;

  /* assert slave select */
  FLASH_SCS = 0;
}

/*****************************************************************************
Name:    SPI_close
Parameters:  None
Returns:  None
Description: Disable UART 0 for SPI and disable slave select
*****************************************************************************/
void SPI_close(void)
{
  /* disable slave select */
  FLASH_SCS = 1;

  /* disable transmit and receive */
  re_u0c1 = 0;
  te_u0c1 = 0;
}

/*****************************************************************************
Name:        SPI_send_receive
Parameters:
      pSrc - pointer to data buffer with data to be transmitted. If NULL, const 0xFF as source.
      pDest - pointer to location to put the received data (can be same as pSrc if desired). 
              If NULL, receive data discarded.
      usBytes - number of bytes to be sent/received
Returns:     none
Description: Performs SPI transfers.
*****************************************************************************/
void SPI_send_receive(unsigned char const *pSrc, unsigned char *pDest, unsigned short usBytes )
{
  unsigned short i;

  if(pSrc == NULL)
    /* read only */
    for(i = 0; i < usBytes ; i++, pDest++)
    {
      u0tbl = 0xFF;
      while( !ri_u0c1 ) ;
      *pDest = u0rbl;
    }
  else if (pDest == NULL)
    /* write only */
    for(i = 0; i < usBytes ; i++, pSrc++)
    {
      u0tbl = *pSrc;
      while( !ri_u0c1 ) ;
      dummy = u0rbl;
    }
  else
    /* read/write */
    for(i = 0; i < usBytes ; i++, pDest++, pSrc++)
    {
      u0tbl = *pSrc;
      while( !ri_u0c1 ) ;
      *pDest = u0rbl;
    }
}


