/**************************************************************************************************
 *
 *  FILE        :StreamingAudio.c
 *  DATE        :Mon, Jul 30, 2007
 *  DESCRIPTION :This code demonstrates streaming an audio file from the serial
 *               flash and playing it using the PWMs.  The way the user selects
 *               which file to listen to is by selecting the file from the
 *               audio_file_array array.  Once a file is selected and found in
 *               the serial flash, it is set to RUNNING.  In this state, the
 *               AudioSample ISR will take care of getting the file's data, decoding
 *               it, and outputting it on the PWMs.
 *
 *               In this code, everytime the user presses SW1 a different audio file
 *               is chosen.  To play 2 channels at once, press SW1 while a file
 *               is already playing.  Once all the files have been played,
 *               it will start back at the first file.
 *
 *               The audio files are stored in the serial flash with a table at the
 *               beginning which tells the audio file's name, location, and size.
 *               24 bytes are used for the name, which is in ascii hex with a 0x00
 *               designating the end.  After these 24 bytes, there are 4 bytes for
 *               file's location (in hex) and 4 bytes for the file's size (in hex).
 *               An example table entry is shown below:
 *
 *               67 72 65 65 6E 00 FF FF FF FF FF FF FF FF FF FF   ('green' 0x00 then all 0xFF's)
 *               FF FF FF FF FF FF FF FF 00 00 04 00 00 00 0B DE   (location = 0x400, size = 0xBDE)
 *
 *               The end of the table is detected when a 0xFF is encountered for
 *               the first lettern in the name.
 *
 *  Copyright:   2007 Renesas Technology Corporation.
 *               All Rights Reserved
 *
 *************************************************************************************************/
#include "FlashSerial.h"                      /* support interface to serial flash */
#include "adpcm4.h"
#include "AudioInfo.h"                        /* MCU specific information for audio */

#define CHANNEL_SAMPLES 32                    /* Size of audio buffers (uncompressed data bytes) */
typedef struct
{
  signed short PCMBuffer[2][CHANNEL_SAMPLES]; /* stores decompressed data (double buffered)*/

  enum {IDLE = 0x00, RUNNING} state;          /* identifies the current state */
  unsigned long location;                     /* starting address for audio in serial flash */
  unsigned long size;                         /* size of compressed file */
  unsigned long position;                     /* current position in file */
  signed short *output;                       /* current sample being output on PWM */

  int volatile *PWM_REG;                      /* which PWM to output from (pointer to sfr) */
  T_ADPCM adpcm;                              /* pointer to object to decode the compressed data */
}tAudioChannel;

#define CHANNEL_COUNT 2                       /* Number of audio channels (1=mono, 2=stereo, etc.) */
/* stores all the audio channels */
struct
{
  tAudioChannel channel[CHANNEL_COUNT];       /* store channel information */
  unsigned char sample;                       /* index into PCM buffers */
  unsigned char buffer_select;                /* select which buffer is active */
  unsigned char sf_flag;                      /* indicates that the serial flash is being used */
} Audio;


#define NumAudioFiles (sizeof(audio_file_array)/sizeof(audio_file_array[0]))
/* This array holds the possible audio files in the current mot file */
char far * const audio_file_array[] =
{
"do not connect",
"green",
"LED is on",
"left",
"light sensor",
"orange",
"please press",
"please verify",
"pushbutton",
"red",
"yellow",
"right",
#if AUDIO_MCU_TYPE == M16C_MCU
  "HTSdemo"
#elif AUDIO_MCU_TYPE == R8C_MCU
  "tau"
#elif AUDIO_MCU_TYPE == H8_MCU
  "quark"
#endif
};


/************* Function Prototypes ***********************************/
void main( void );
char FlashFind( char far *name,  unsigned char channel);
char ADPCM_Start(char far *name, unsigned char channel);
void delay(int count);
/* From HWInit.c */
void InitCLOCK( void );
void InitAudio(void);
void PortInit(void);

/*****************************************************************************
Name:           AudioSample
Parameters:     none
Returns:        none
Description:    Outputs audio data through the PWMs.  This ISR also decodes
                the compressed data.  It will get more data and decode it for
                one channel per interrupt request.
*****************************************************************************/
unsigned char interrupt_count = 0;
void AudioSample(void)
{
  /* write new data to PWMs */
  *Audio.channel[0].PWM_REG = *Audio.channel[0].output;
  *Audio.channel[1].PWM_REG = *Audio.channel[1].output;

  #if AUDIO_MCU_TYPE == H8_MCU || AUDIO_MCU_TYPE == R8C_MCU
    /* clear interrupt flag so we can re-interrupt */
    TIMER_IRQ_FLAG = 0;
  #endif

  #if AUDIO_MCU_TYPE == H8_MCU || AUDIO_MCU_TYPE == R8C_MCU

      interrupt_count++;

      //if >= AUDIO_PWM_SKIPS then it is time for next sample to be output
      if(interrupt_count < AUDIO_PWM_SKIPS){
        return;
      }

      interrupt_count = 0;
  #endif

  /* re-enable interrupts to allow sample output while reading */
  ENABLE_IRQ;

  /* Update the PWMs */
  Audio.channel[0].output++;
  Audio.channel[1].output++;

  /* point to next sample */
  Audio.sample++;

  /* check if we need to extract more samples...return if not */
  if (Audio.sample < CHANNEL_SAMPLES)
  {
    return;
  }

  /* this makes sure that multiple interrupts do not read the serial flash at the same time */
  if(Audio.sf_flag == 1)
  {
      Audio.sample = Audio.sample - 1;
      return;
  }

  /* read more data from flash and expand */
  {
    tAudioChannel *pC=Audio.channel;
    static unsigned char SPI_buffer[CHANNEL_SAMPLES/2];
    unsigned char ci;

    /* set this flag so that another interrupt cannot come in this section */
    Audio.sf_flag = 1;

    /* switch active buffers */
    Audio.buffer_select = (Audio.buffer_select +1) & 0x01;

    /* reset our index counter */
    Audio.sample = 0;
    for(ci = 0; ci < CHANNEL_COUNT; ci++){
      Audio.channel[ci].output = Audio.channel[ci].PCMBuffer[Audio.buffer_select];    // Output Channel Data
    }


    #if AUDIO_MCU_TYPE == M16C_MCU || AUDIO_MCU_TYPE == R8C_MCU
      /* raise IPL so we can re-interrupt */
      TIMER_INT_REG = TIMER_IPL+1;
    #endif

    /* read/expand more data into buffers */
    for (ci=0; ci < CHANNEL_COUNT; ci++, pC++)
    {
      signed short *pS = pC->PCMBuffer[((Audio.buffer_select+1)&0x01)];
      unsigned char iS;
      /* check if more data still in file */
      if (pC->state == RUNNING)
      {
        /* read buffer worth of data in */
        FlashReadData( pC->location + pC->position, SPI_buffer, sizeof(SPI_buffer));
        pC->position += sizeof(SPI_buffer);


        //decompress the data so that it can be output
        pC->adpcm.output = pS;
        pC->adpcm.input = SPI_buffer;
        pC->adpcm.size =  CHANNEL_SAMPLES;
        adpcm_decode4(&(pC->adpcm));

        /* check if file completed */
        if (pC->position >= pC->size)
        {
          pC->state = IDLE;
          /* clear any superfluous samples */
          for (iS=CHANNEL_SAMPLES-((pC->position-pC->size)*2); iS < CHANNEL_SAMPLES; iS++)
            pS[iS] = 0;
        }

        /* scale the data */
        for ( iS=0; iS<CHANNEL_SAMPLES; iS++, pS++)
        {
          *pS += 0x8000;
          #if AUDIO_MCU_TYPE == M16C_MCU
            *pS &= 0xFF00;
            if (*pS == 0xFF00)
                *pS = 0xFE00;
          #elif AUDIO_MCU_TYPE == R8C_MCU || AUDIO_MCU_TYPE == H8_MCU
            *pS = *pS>>(16-AUDIO_PWM_SIZE);                          //shift lower bits off
            *pS &= (0x1 << AUDIO_PWM_SIZE)-1;                        //mask off the upper 8 bits
            *pS += AUDIO_PWM_SILENCE-(0x1 << (AUDIO_PWM_SIZE-1));
          #endif
        }
      }
      else
      {
        /* zero the buffer for silence and go to IDLE */
        for (iS=0; iS < CHANNEL_SAMPLES; iS++, pS++)
          *pS = AUDIO_PWM_SILENCE;
      }
    }

    #if AUDIO_MCU_TYPE == M16C_MCU || AUDIO_MCU_TYPE == R8C_MCU
      /* lower IPL to normal */
      TIMER_INT_REG = TIMER_IPL;
    #endif

    /* lower flag and let next interrupt in to read the serial flash */
    Audio.sf_flag = 0;

  }
}

/*****************************************************************************
Name:        ADPCM_Start
Parameters:  name   -   name of file to search for in serial flash
             channel-   which channel to assign this sound too
Returns:     3 = Channel currently RUNNING
             2 = Not valid channel
             1 = File Not Found
             0 = Success (file was found and started)
Description: Starts an audio file on a given channel.
*****************************************************************************/
char ADPCM_Start(char far *name, unsigned char channel)
{
    char i = 0;
    /* make sure it is a valid channel */
    if(channel < CHANNEL_COUNT){

        /* make sure channel is not already in use */
        if(Audio.channel[channel].state == IDLE){

            /* find file in serial flash */
            if(FlashFind(name, channel) == 1){
                /* setup channel */
                adpcm_init4(&Audio.channel[channel].adpcm);                                /* initialize channel for decoding */
                Audio.channel[channel].position = 0;                                       /* init position in file to the beginning */
                switch(channel){
                    case 0: Audio.channel[channel].PWM_REG = AUDIO_PWM_CH_1;
                            break;
                    case 1: Audio.channel[channel].PWM_REG = AUDIO_PWM_CH_2;
                            break;
                    default:
                            break;
                }
                Audio.channel[channel].state = RUNNING;
                return 0;

            } else {
                return 1;
            }

        } else {
            return 3;
        }

    } else {
        return 2;
    }
}

/*****************************************************************************
Name:        FlashFind
Parameters:  name   -   name of file to search for in serial flash
             channel-   which channel of audio to use
Returns:     1 = Success (file was found)
             0 = File Not Found
Description: Searches the external serial flash for the designated audio
             file.  If it is found then it returns Success and will give
             the size and location of the file.
*****************************************************************************/
char FlashFind( char far *name,  unsigned char channel)
{
  unsigned long address = 0;
  unsigned char i = 0;
  unsigned char status;
  static unsigned char SPI_buffer[32];


  //Each audio file header is 32 bytes.  The first time that a 0xFF is found
  //in the 'name' field, the search will stop if the name has not already
  //been found.
  for (; ; )
  {
    //read in buffer from the serial flash
    status = FlashReadData(address, SPI_buffer, sizeof(SPI_buffer));
    if(status != 0)
      return 0;
    address += sizeof(SPI_buffer);

    //i is used for the current position inside the current audio header
    i = 0;

    if((SPI_buffer[0] == 0xFF))    //this means there are no more files to search
      return 0;

    //the OR with 0x20 is to make sure they characters are lowercase
    //It should be noted that with this method filenames should only be letters or numbers.
    while((SPI_buffer[i] | 0x20) == (name[i] | 0x20))  //keep looping while characters are the same
    {
      if(SPI_buffer[i] == 0)        //there was a complete name match
      {
        //fill in information for location of audio file, and its size
        Audio.channel[channel].location = (((unsigned long)SPI_buffer[24])      |
                                      ((unsigned long)SPI_buffer[25] << 8)  |
                                      ((unsigned long)SPI_buffer[26] << 16) |
                                      ((unsigned long)SPI_buffer[27] << 24) );
        Audio.channel[channel].size = (((unsigned long)SPI_buffer[28])      |
                                      ((unsigned long)SPI_buffer[29] << 8)  |
                                      ((unsigned long)SPI_buffer[30] << 16) |
                                      ((unsigned long)SPI_buffer[31] << 24) );
        return 1;
      }
      i++;
    }
  }
  return 0;
}

/*****************************************************************************
Name:           main
Parameters:     none
Returns:        none
Description:    State machine to test Sub-Atomic Particle Hardware
*****************************************************************************/
void main( void )
{
  unsigned char TestState=0;
  unsigned int audio_index = 0;

  DISABLE_IRQ

  /* Initialize the system clocks */
  InitCLOCK();
  /* Setup the ADC */
  InitAD();
  /* Initialize the board for audio output */
  InitAudio();

  while(1)
  {
    if (SW1 == 0)
    {
      while (SW1 == 0) ;
      TestState++;
    }

    switch (TestState)
    {
      default:
      {
        TestState++;
        /* All off condition */

        /* configure ports */
        PortInit();

        delay(5000);

        break;
      }

      case 2:
      {
        InitSubClockTest();
        TestState++;
        break;
      }

      case 3:
      {
        TestADC();

          DISABLE_IRQ
          ADPCM_Start(audio_file_array[0], 0);
          ADPCM_Start(audio_file_array[0], 1);
          ENABLE_IRQ

        break;
      }


      case 4:
      {
        char i;

        // wait for channels to complete processing before switching
        for(i=0; i<CHANNEL_COUNT; i++){

            /* If this channel is already running then try the next */
            if (Audio.channel[i].state != IDLE)
                continue;

            DISABLE_IRQ
            //searches for the given filename in the serial flash
            ADPCM_Start(audio_file_array[audio_index], i);
            ENABLE_IRQ

            audio_index++;
            if(audio_index >= NumAudioFiles)
                audio_index = 1;

            break;
        }

        TestState--;
        break;
      }

    }
  }
}


void delay(int count)
{
  while (count > 0)
    count--;
}
